﻿'A reference to the Phidget22.NET.dll must be added to new projects in VB.Net 
'From the "Project" menu select "Add Reference" 
'From the "Add Reference" window select the "Browse" tab 
'Browse to the directory you installed the Phidgets library to 
'The default is "C:\Program Files\Phidgets" 
'Select "Phidget22.NET.dll" and click "OK" 
'To view objects that have been added, select the "View" menu 
'Select "Object Browser" 
'Then expand the Phidget22.NET.dll item 



Imports Phidget22
Imports Phidget22.Events

Public Class Form1
    Dim settingsBox As SettingsForm = Nothing
    Structure defaultMotorSettings
        Public Kp As Double
        Public Ki As Double
        Public Kd As Double
        Public acceleration As Double
        Public velocityLimit As Double
        Public rescaleFactor As Double
        Public deadband As Double
        Public currentLimit As Double
        Public currentRegulatorGain As Double
        Public stallVelocity As Double
        Public encoderIOMode As EncoderIOMode
    End Structure

    Dim defaults As defaultMotorSettings
    Dim dataPoints As Integer

    Dim WithEvents device As Phidget22.MotorPositionController = Nothing
    Dim open As ExampleUtils.ExampleUtils.CommandLineOpen
    Dim errorBox As ErrorEventBox

    Public Sub New()
        open = New ExampleUtils.ExampleUtils.CommandLineOpen(Me)
        Phidget22.Phidget.InvokeEventCallbacks = True
        InitializeComponent()
    End Sub

    Public Sub New(ByVal commandLine As String())
        open = New ExampleUtils.ExampleUtils.CommandLineOpen(Me)
        open.commandLine = commandLine
        Phidget22.Phidget.InvokeEventCallbacks = True
        InitializeComponent()
    End Sub

#Region "Form Events"
    Private Sub Form1_Load(sender As Object, e As EventArgs) Handles MyBase.Load
        errorBox = New ErrorEventBox(Me)
        device = New Phidget22.MotorPositionController()
        settingsBox = New SettingsForm(Me)

        open.openCmdLine(device)
    End Sub

    Private Sub Form1_FormClosing(sender As Object, e As FormClosingEventArgs) Handles MyBase.FormClosing
        If device.Attached Then
            device.Close()
        End If
    End Sub

#End Region

#Region "Motor events"
    Public Sub motor_Attach(ByVal sender As Object, ByVal e As AttachEventArgs) Handles device.Attach
        Dim attached As Phidget22.MotorPositionController = CType(sender, Phidget22.MotorPositionController)
        PhidgetInfoBox1.FillPhidgetInfo(CType(sender, Phidget))

        'grab default values
        Try
            defaults.Kp = attached.Kp
            defaults.Ki = attached.Ki
            defaults.Kd = attached.Kd
            defaults.acceleration = attached.Acceleration
            defaults.velocityLimit = attached.VelocityLimit
            defaults.rescaleFactor = attached.RescaleFactor
            defaults.deadband = attached.DeadBand
            If attached.DeviceID = DeviceID.PN_DCC1000 Or attached.DeviceID = DeviceID.PN_DCC1001 Then
                defaults.currentLimit = attached.CurrentLimit
            End If
            If attached.DeviceID = DeviceID.PN_DCC1000 Then
                defaults.currentRegulatorGain = attached.CurrentRegulatorGain
                defaults.encoderIOMode = attached.IOMode
            End If
            If attached.DeviceID = DeviceID.PN_DCC1100 Then
                defaults.stallVelocity = attached.StallVelocity
            End If
        Catch ex As PhidgetException
            errorBox.addMessage("Error getting device defaults: " + ex.Message())
        End Try

        'try saved settings
        applySavedSettings(attached, Nothing)

        'change data interval
        attached.DataInterval = attached.MinDataInterval

        'set up "other motor settings" box, which is initially hidden
        Select Case attached.DeviceID
            Case DeviceID.PN_DCC1000
                currentLimPanel.Visible = True
                currentRegPanel.Visible = True
                ioModePanel.Visible = True
                fanModePanel.Visible = True
                currentLimitText.Minimum = Convert.ToDecimal(attached.MinCurrentLimit)
                currentLimitText.Maximum = Convert.ToDecimal(attached.MaxCurrentLimit)
                currentLimitText.Value = Convert.ToDecimal(attached.CurrentLimit)
                currentRegText.Value = Convert.ToDecimal(attached.CurrentRegulatorGain)
                For Each eio As EncoderIOMode In [Enum].GetValues(GetType(EncoderIOMode))
                    encoderIOCombo.Items.Add(eio)
                Next
                encoderIOCombo.SelectedIndex = 0

                For Each fm As FanMode In [Enum].GetValues(GetType(FanMode))
                    fanModeCombo.Items.Add(fm)
                Next
                fanModeCombo.SelectedIndex = 2

                additionalSettings.Controls.SetChildIndex(currentLimPanel, 0)
                additionalSettings.Controls.SetChildIndex(currentRegPanel, 1)
                additionalSettings.Controls.SetChildIndex(ioModePanel, 2)
                additionalSettings.Controls.SetChildIndex(fanModePanel, 3)
            Case DeviceID.PN_DCC1001
                currentLimPanel.Visible = True
                currentLimitText.Minimum = Convert.ToDecimal(attached.MinCurrentLimit)
                currentLimitText.Maximum = Convert.ToDecimal(attached.MaxCurrentLimit)
                currentLimitText.Value = Convert.ToDecimal(attached.CurrentLimit)

                additionalSettings.Controls.SetChildIndex(currentLimPanel, 0)
            Case DeviceID.PN_DCC1100
                stallVelocityPanel.Visible = True
                stallVelocityText.Value = Convert.ToDecimal(attached.StallVelocity)
                additionalSettings.Controls.SetChildIndex(stallVelocityPanel, 0)
        End Select
    End Sub

    Public Sub motor_Detach(ByVal sender As Object, ByVal e As DetachEventArgs) Handles device.Detach
        PhidgetInfoBox1.Clear()
        clearGraphPoints()
        currentLimPanel.Visible = False
        currentRegPanel.Visible = False
        ioModePanel.Visible = False
        fanModePanel.Visible = False
        stallVelocityPanel.Visible = False
        setEngaged.Text = "Engage"
        kpText.Text = "0"
        kiText.Text = "0"
        kdText.Text = "0"
        targetPositionText.Text = "0"
        accelerationText.Text = "0"
        velocityText.Text = "0"
        rescaleText.Text = "?"
        deadBandText.Text = "?"
        positionLabel.Text = "0"
        dutyCycleLabel.Text = "0"
    End Sub

    Public Sub motor_error(ByVal sender As Object, ByVal e As ErrorEventArgs) Handles device.Error
        errorBox.addMessage("Error " + e.Code + ": " + e.Description)
        If e.Code = ErrorEventCode.MotorStallCondition Then 'this error will disengage the motor
            setEngaged.Text = "Engage Motor"
        End If
    End Sub

    Public Sub motor_position_change(ByVal sender As Object, ByVal e As MotorPositionControllerPositionChangeEventArgs) Handles device.PositionChange
        PositionChart.Series("MotorPosition").Points.AddY(e.Position)
        PositionChart.Series("TargetPosition").Points.AddY(device.TargetPosition)
        positionLabel.Text = e.Position.ToString("F3")
        While PositionChart.Series("MotorPosition").Points.Count > dataPoints
            PositionChart.Series("MotorPosition").Points.RemoveAt(0)
            PositionChart.Series("TargetPosition").Points.RemoveAt(0)
        End While
        PositionChart.ChartAreas(0).RecalculateAxesScale()
    End Sub

    Public Sub motor_dutycycle_update(ByVal sender As Object, ByVal e As MotorPositionControllerDutyCycleUpdateEventArgs) Handles device.DutyCycleUpdate
        dutyCycleLabel.Text = e.DutyCycle.ToString()
    End Sub
#End Region

#Region "graph/misc functions"
    Sub updateRescaleProperties(ByVal ch As Phidget22.MotorPositionController)
        accelerationText.Increment = updateLimits(ch.Acceleration)
        velocityText.Increment = updateLimits(ch.VelocityLimit)
        targetPositionText.Minimum = Convert.ToDecimal(ch.MinPosition)
        targetPositionText.Maximum = Convert.ToDecimal(ch.MaxPosition)
        If ch.DeviceID = DeviceID.PN_DCC1100 Then
            stallVelocityText.Increment = updateLimits(ch.StallVelocity)
            stallVelocityText.TextAlign = ch.StallVelocity.ToString()
        End If

        accelerationText.Text = ch.Acceleration.ToString()
        velocityText.Text = ch.VelocityLimit.ToString()
        deadBandText.Text = ch.DeadBand.ToString()
    End Sub

    Sub updateGUI(ByVal ch As Phidget22.MotorPositionController)
        'fill in form with info from device
        kpText.Value = Convert.ToDecimal(ch.Kp)
        kiText.Value = Convert.ToDecimal(ch.Ki)
        kdText.Value = Convert.ToDecimal(ch.Kd)
        rescaleText.Text = ch.RescaleFactor.ToString()
        If ch.DeviceID = DeviceID.PN_DCC1000 Or ch.DeviceID = DeviceID.PN_DCC1001 Then
            currentLimitText.Value = Convert.ToDecimal(ch.CurrentLimit)
        End If
        If ch.DeviceID = DeviceID.PN_DCC1000 Then
            currentRegText.Value = Convert.ToDecimal(ch.CurrentRegulatorGain)
        End If
        If ch.DeviceID = DeviceID.PN_DCC1100 Then
            stallVelocityText.Value = Convert.ToDecimal(ch.StallVelocity)
        End If
        updateRescaleProperties(ch)

        'set numeric updown limits
        kpText.Increment = updateLimits(ch.Kp)
        kiText.Increment = updateLimits(ch.Ki)
        kdText.Increment = updateLimits(ch.Kd)
    End Sub

    Sub applySavedSettings(ByVal ch As Phidget22.MotorPositionController, ByVal collection As CheckedListBox.CheckedItemCollection)
        If collection Is Nothing Then
            Try
                'set rescale because it will impact other properties
                If My.Settings.isRescaleFactorSet Then
                    ch.RescaleFactor = My.Settings.Default.rescaleFactor
                End If

                If My.Settings.isKpSet Then
                    ch.Kp = My.Settings.Kp
                End If

                If My.Settings.isKiSet Then
                    ch.Ki = My.Settings.Ki
                End If

                If My.Settings.isKdSet Then
                    ch.Kd = My.Settings.Kd
                End If

                If My.Settings.isAccelerationSet Then
                    ch.Acceleration = My.Settings.Acceleration
                End If

                If My.Settings.isVelocityLimitSet Then
                    ch.VelocityLimit = My.Settings.VelocityLimit
                End If

                If My.Settings.isDeadbandSet Then
                    ch.DeadBand = My.Settings.DeadBand
                End If

                If My.Settings.isCurrentLimitSet And (ch.DeviceID = DeviceID.PN_DCC1000 Or ch.DeviceID = DeviceID.PN_DCC1001) Then
                    ch.CurrentLimit = My.Settings.CurrentLimit
                End If

                If My.Settings.isEncoderIOModeSet And ch.DeviceID = DeviceID.PN_DCC1000 Then
                    ch.IOMode = My.Settings.encoderIOMode
                End If

                If My.Settings.isCurrentRegulatorGainSet And ch.DeviceID = DeviceID.PN_DCC1000 Then
                    ch.CurrentRegulatorGain = My.Settings.CurrentRegulatorGain
                End If

                If My.Settings.isStallVelocitySet And ch.DeviceID = DeviceID.PN_DCC1100 Then
                    ch.StallVelocity = My.Settings.StallVelocity
                End If

                If My.Settings.isDataPointsSet Then
                    dataPoints = My.Settings.dataPoints
                    dataPointsText.Text = dataPoints.ToString()
                Else
                    dataPoints = 100
                    dataPointsText.Text = "100"
                End If
            Catch ex As PhidgetException
                errorBox.addMessage("Error setting saved parameter: " + ex.Message)
            End Try
        Else
            Try
                For i As Integer = 0 To collection.Count - 1
                    If collection(i).ToString() = "Kp" Then
                        ch.Kp = My.Settings.Kp
                    ElseIf collection(i).ToString() = "Ki"
                        ch.Ki = My.Settings.Ki
                    ElseIf collection(i).ToString() = "Kd"
                        ch.Kd = My.Settings.Kd
                    ElseIf collection(i).ToString() = "Acceleration"
                        ch.Acceleration = My.Settings.Acceleration
                    ElseIf collection(i).ToString() = "Velocity Limit"
                        ch.VelocityLimit = My.Settings.VelocityLimit
                    ElseIf collection(i).ToString() = "Rescale Factor"
                        ch.RescaleFactor = My.Settings.rescaleFactor
                    ElseIf collection(i).ToString() = "Deadband"
                        ch.DeadBand = My.Settings.DeadBand
                    ElseIf collection(i).ToString() = "Current Limit"
                        ch.CurrentLimit = My.Settings.CurrentLimit
                    ElseIf collection(i).ToString() = "Current Regulator Gain"
                        ch.CurrentRegulatorGain = My.Settings.CurrentRegulatorGain
                    ElseIf collection(i).ToString() = "Stall Velocity"
                        ch.StallVelocity = My.Settings.StallVelocity
                    End If
                Next
            Catch ex As PhidgetException
                errorBox.addMessage("Error getting default values: " + ex.Message)
            End Try
        End If

        updateGUI(ch)
    End Sub

    Sub setDefaults(ByVal ch As Phidget22.MotorPositionController, ByVal collection As CheckedListBox.CheckedItemCollection)
        Try
            For i As Integer = 0 To collection.Count - 1
                If collection(i).ToString() = "Kp" Then
                    ch.Kp = defaults.Kp
                ElseIf collection(i).ToString() = "Ki"
                    ch.Ki = defaults.Ki
                ElseIf collection(i).ToString() = "Kd"
                    ch.Kd = defaults.Kd
                ElseIf collection(i).ToString() = "Acceleration"
                    ch.Acceleration = defaults.acceleration
                ElseIf collection(i).ToString() = "Velocity Limit"
                    ch.VelocityLimit = defaults.velocityLimit
                ElseIf collection(i).ToString() = "Rescale Factor"
                    ch.RescaleFactor = defaults.rescaleFactor
                ElseIf collection(i).ToString() = "Deadband"
                    ch.DeadBand = defaults.deadband
                ElseIf collection(i).ToString() = "Current Limit"
                    ch.CurrentLimit = defaults.currentLimit
                ElseIf collection(i).ToString() = "Current Regulator Gain"
                    ch.CurrentRegulatorGain = defaults.currentRegulatorGain
                ElseIf collection(i).ToString() = "Stall Velocity"
                    ch.StallVelocity = defaults.stallVelocity
                ElseIf collection(i).ToString() = "Encoder IO Mode"
                    ch.IOMode = defaults.encoderIOMode
                End If
            Next
        Catch ex As PhidgetException
            errorBox.addMessage("Error getting default values: " + ex.Message)
        End Try

        updateGUI(ch)
    End Sub

    Private Function updateLimits(ByVal n As Double) As Decimal
        n = Math.Abs(n)
        If n <> 0 And n > 1 Then
            Return Convert.ToDecimal(Math.Pow(10, (Math.Floor(Math.Log10(n)))) / 2)
        Else
            Return Convert.ToDecimal(0.5)
        End If
    End Function

    Private Sub clearGraphPoints()
        For Each series In PositionChart.Series
            series.Points.Clear()
        Next
    End Sub

    Private Sub clearGraph_Click(sender As Object, e As EventArgs) Handles clearGraph.Click
        clearGraphPoints()
    End Sub
#End Region

#Region "GUI controls"
    Private Sub setRescaleFactor_Click(sender As Object, e As EventArgs) Handles setRescaleFactor.Click
        Dim result As Double
        If Double.TryParse(rescaleText.Text, result) Then
            Try
                device.RescaleFactor = result
            Catch ex As PhidgetException
                errorBox.addMessage("Error setting rescale factor: " + ex.Message)
            End Try
        End If

        updateRescaleProperties(device)
    End Sub

    Private Sub setDeadband_Click(sender As Object, e As EventArgs) Handles setDeadband.Click
        Dim result As Double
        If Double.TryParse(deadBandText.Text, result) Then
            Try
                device.DeadBand = result
            Catch ex As PhidgetException
                errorBox.addMessage("Error setting deadband: " + ex.Message)
            End Try
        End If
    End Sub

    Private Sub setEngaged_Click(sender As Object, e As EventArgs) Handles setEngaged.Click
        Try
            If device.Engaged Then
                device.Engaged = False
                setEngaged.Text = "Engage Motor"
            Else
                device.Engaged = True
                setEngaged.Text = "Disengage Motor"
            End If
        Catch ex As PhidgetException
            errorBox.addMessage("Error setting engaged: " + ex.Message)
        End Try
    End Sub

    Private Sub zeroPosition_Click(sender As Object, e As EventArgs) Handles zeroPosition.Click
        Try
            device.AddPositionOffset(-1 * device.Position)
            'clear graph
            clearGraphPoints()
        Catch ex As PhidgetException
            errorBox.addMessage("Error zeroing position" + ex.Message)
        End Try
    End Sub



#Region "targetPosition"
    Private Sub incrementPositionText_TextChanged(sender As Object, e As EventArgs) Handles incrementPositionText.TextChanged
        incrementPosition()
    End Sub

    Private Sub incrementPositionText_KeyDown(sender As Object, e As KeyEventArgs) Handles incrementPositionText.KeyDown
        If e.KeyCode = Keys.Enter Then
            e.SuppressKeyPress = True
            incrementPosition()
        End If
    End Sub

    Private Sub incrementPosition()
        Dim result As Double
        If Double.TryParse(incrementPositionText.Text, result) Then
            If result > 0 Then 'cant have negative increment value
                targetPositionText.Increment = Convert.ToDecimal(result)
            End If
        End If
    End Sub

    Private Sub targetPositionText_ValueChanged(sender As Object, e As EventArgs) Handles targetPositionText.ValueChanged
        If device.Attached Then 'prevent text change event from happening when resetting textbox on detach
            setTargetPosition()
        End If
    End Sub

    Private Sub targetPositionText_KeyDown(sender As Object, e As KeyEventArgs) Handles targetPositionText.KeyDown
        If e.KeyCode = Keys.Enter Then
            e.SuppressKeyPress = True
            setTargetPosition()
        End If
    End Sub

    Private Sub setTargetPosition()
        Try
            device.TargetPosition = Convert.ToDouble(targetPositionText.Value)
        Catch ex As PhidgetException
            errorBox.addMessage("Error setting target position: " + ex.Message)
        End Try
    End Sub


#End Region

#Region "velocity"
    Private Sub velocityText_ValueChanged(sender As Object, e As EventArgs) Handles velocityText.ValueChanged
        If device.Attached Then 'prevent text change event from happening when resetting text box on detach
            setVelocity()
        End If
    End Sub

    Private Sub velocityText_KeyDown(sender As Object, e As KeyEventArgs) Handles velocityText.KeyDown
        If e.KeyCode = Keys.Enter Then
            e.SuppressKeyPress = True
            setVelocity()
        End If
    End Sub

    Private Sub setVelocity()
        Try
            device.VelocityLimit = Convert.ToDouble(velocityText.Value)
            velocityText.Increment = updateLimits(device.VelocityLimit)
        Catch ex As PhidgetException
            errorBox.addMessage("Error setting acceleration: " + ex.Message)
        End Try
    End Sub


#End Region

#Region "acceleration"
    Private Sub accelerationText_ValueChanged(sender As Object, e As EventArgs) Handles accelerationText.ValueChanged
        If device.Attached Then 'prevent text change event from happening when resetting text box on detach
            setAcceleration()
        End If
    End Sub

    Private Sub accelerationText_KeyDown(sender As Object, e As KeyEventArgs) Handles accelerationText.KeyDown
        If e.KeyCode = Keys.Enter Then
            e.SuppressKeyPress = True
            setAcceleration()
        End If
    End Sub

    Private Sub setAcceleration()
        Try
            device.Acceleration = Convert.ToDouble(accelerationText.Value)
            accelerationText.Increment = updateLimits(device.Acceleration)
        Catch ex As PhidgetException
            errorBox.addMessage("Error setting acceleration: " + ex.Message)
        End Try
    End Sub


#End Region

#Region "kp"
    Private Sub kpText_ValueChanged(sender As Object, e As EventArgs) Handles kpText.ValueChanged
        If device.Attached Then 'prevent text change event from happening when resetting text box on detach
            setKp()
        End If
    End Sub

    Private Sub kpText_KeyDown(sender As Object, e As KeyEventArgs) Handles kpText.KeyDown
        If e.KeyCode = Keys.Enter Then
            e.SuppressKeyPress = True
            setKp()
        End If
    End Sub

    Private Sub setKp()
        Try
            device.Kp = Convert.ToDouble(kpText.Value)
            kpText.Increment = updateLimits(device.Kp)
        Catch ex As PhidgetException
            errorBox.addMessage("Error setting acceleration: " + ex.Message)
        End Try
    End Sub
#End Region

#Region "Ki"
    Private Sub KiText_ValueChanged(sender As Object, e As EventArgs) Handles kiText.ValueChanged
        If device.Attached Then 'prevent text change event from happening when resetting text box on detach
            setKi()
        End If
    End Sub

    Private Sub KiText_KeyDown(sender As Object, e As KeyEventArgs) Handles kiText.KeyDown
        If e.KeyCode = Keys.Enter Then
            e.SuppressKeyPress = True
            setKi()
        End If
    End Sub

    Private Sub setKi()
        Try
            device.Ki = Convert.ToDouble(kiText.Value)
            kiText.Increment = updateLimits(device.Ki)
        Catch ex As PhidgetException
            errorBox.addMessage("Error setting acceleration: " + ex.Message)
        End Try
    End Sub
#End Region

#Region "Kd"
    Private Sub KdText_ValueChanged(sender As Object, e As EventArgs) Handles kdText.ValueChanged
        If device.Attached Then 'prevent text change event from happening when resetting text box on detach
            setKd()
        End If
    End Sub

    Private Sub KdText_KeyDown(sender As Object, e As KeyEventArgs) Handles kdText.KeyDown
        If e.KeyCode = Keys.Enter Then
            e.SuppressKeyPress = True
            setKd()
        End If
    End Sub

    Private Sub setKd()
        Try
            device.Kd = Convert.ToDouble(kdText.Value)
            kdText.Increment = updateLimits(device.Kd)
        Catch ex As PhidgetException
            errorBox.addMessage("Error setting acceleration: " + ex.Message)
        End Try
    End Sub


#End Region

    Private Sub saveSettings_Click(sender As Object, e As EventArgs) Handles saveSettings.Click
        settingsBox.ShowSettingsBox(0, device.DeviceID)
    End Sub

    Private Sub revertToSavedSettings_Click(sender As Object, e As EventArgs) Handles revertToSavedSettings.Click
        settingsBox.ShowSettingsBox(1, device.DeviceID)
    End Sub

    Private Sub resetSettings_Click(sender As Object, e As EventArgs) Handles resetSettings.Click
        settingsBox.ShowSettingsBox(2, device.DeviceID)
    End Sub


#End Region

#Region "Additional motor settings"
    Private Sub showOtherSettings_CheckedChanged(sender As Object, e As EventArgs) Handles showOtherSettings.CheckedChanged
        Dim check As CheckBox = CType(sender, CheckBox)
        If check.Checked Then
            motorSettingsBox.Visible = True
            Me.Size = New Size(Me.Size.Width + motorSettingsBox.Width + 10, Me.Size.Height)
        Else
            Me.Size = New Size(Me.Size.Width - motorSettingsBox.Width, Me.Size.Height)
            motorSettingsBox.Visible = False
        End If
    End Sub

#Region "current regulator gain"
    Private Sub currentRegText_ValueChanged(sender As Object, e As EventArgs) Handles currentRegText.ValueChanged
        If device IsNot Nothing Then
            setCurrentRegulatorGain()
        End If
    End Sub

    Private Sub currentRegText_KeyDown(sender As Object, e As KeyEventArgs) Handles currentRegText.KeyDown
        If e.KeyCode = Keys.Enter Then
            e.SuppressKeyPress = True
            setCurrentRegulatorGain()
        End If
    End Sub

    Private Sub setCurrentRegulatorGain()
        Try
            device.CurrentRegulatorGain = Convert.ToDouble(currentRegText.Value)
        Catch ex As PhidgetException
            errorBox.addMessage("Error setting current regulator gain: " + ex.Message)
        End Try
    End Sub
#End Region

#Region "current limit"
    Private Sub currentLimitText_ValueChanged(sender As Object, e As EventArgs) Handles currentLimitText.ValueChanged
        If device IsNot Nothing Then
            setCurrentLimit()
        End If
    End Sub

    Private Sub currentLimitText_KeyDown(sender As Object, e As KeyEventArgs) Handles currentLimitText.KeyDown
        If e.KeyCode = Keys.Enter Then
            e.SuppressKeyPress = True
            setCurrentLimit()
        End If
    End Sub

    Private Sub setCurrentLimit()
        Try
            device.CurrentLimit = Convert.ToDouble(currentLimitText.Value)
        Catch ex As PhidgetException
            errorBox.addMessage("Error setting current limit: " + ex.Message)
        End Try
    End Sub
#End Region

#Region "stall velocity"
    Private Sub stallVelocityText_ValueChanged(sender As Object, e As EventArgs) Handles stallVelocityText.ValueChanged
        If device IsNot Nothing Then
            setStallVelocity()
        End If
    End Sub

    Private Sub stallVelocityText_KeyDown(sender As Object, e As KeyEventArgs) Handles stallVelocityText.KeyDown
        If e.KeyCode = Keys.Enter Then
            e.SuppressKeyPress = True
            setStallVelocity()
        End If
    End Sub

    Private Sub setStallVelocity()
        Try
            device.StallVelocity = Convert.ToDouble(stallVelocityText.Value)
        Catch ex As PhidgetException
            errorBox.addMessage("Error setting stall velocity: " + ex.Message)
        End Try
    End Sub


#End Region

    Private Sub fanModeCombo_SelectedIndexChanged(sender As Object, e As EventArgs) Handles fanModeCombo.SelectedIndexChanged
        Try
            device.FanMode = CType(fanModeCombo.Items(fanModeCombo.SelectedIndex), FanMode)
        Catch ex As PhidgetException
            errorBox.addMessage("Error setting fan mode: " + ex.Message)
        End Try
    End Sub

    Private Sub encoderIOCombo_SelectedIndexChanged(sender As Object, e As EventArgs) Handles encoderIOCombo.SelectedIndexChanged
        Try
            device.IOMode = CType(encoderIOCombo.Items(encoderIOCombo.SelectedIndex), EncoderIOMode)
        Catch ex As PhidgetException
            errorBox.addMessage("Error setting IO mode: " + ex.Message)
        End Try
    End Sub

    Public Sub saveOrResetSettings(collection As CheckedListBox.CheckedItemCollection, state As Integer)
        settingsBox.Hide()
        Dim outputText As String = ""
        If state = 0 Then 'save settings
            outputText = "Saved settings!"
            For i As Integer = 0 To collection.Count - 1
                If collection(i).ToString() = "Kp" Then
                    My.Settings.Kp = device.Kp
                    My.Settings.isKpSet = True
                ElseIf collection(i).ToString() = "Ki"
                    My.Settings.Ki = device.Ki
                    My.Settings.isKiSet = True
                ElseIf collection(i).ToString() = "Kd"
                    My.Settings.Kd = device.Kd
                    My.Settings.isKdSet = True
                ElseIf collection(i).ToString() = "Acceleration"
                    My.Settings.Acceleration = device.Acceleration
                    My.Settings.isAccelerationSet = True
                ElseIf collection(i).ToString() = "Velocity Limit"
                    My.Settings.VelocityLimit = device.VelocityLimit
                    My.Settings.isVelocityLimitSet = True
                ElseIf collection(i).ToString() = "Rescale Factor"
                    My.Settings.rescaleFactor = device.RescaleFactor
                    My.Settings.isRescaleFactorSet = True
                ElseIf collection(i).ToString() = "Deadband"
                    My.Settings.DeadBand = device.DeadBand
                    My.Settings.isDeadbandSet = True
                ElseIf collection(i).ToString() = "Current Limit"
                    My.Settings.CurrentLimit = device.CurrentLimit
                    My.Settings.isCurrentLimitSet = True
                ElseIf collection(i).ToString() = "Current Regulator Gain"
                    My.Settings.CurrentRegulatorGain = device.CurrentRegulatorGain
                    My.Settings.isCurrentRegulatorGainSet = True
                ElseIf collection(i).ToString() = "Stall Velocity"
                    My.Settings.StallVelocity = device.StallVelocity
                    My.Settings.isStallVelocitySet = True
                ElseIf collection(i).ToString() = "Encoder IO Mode"
                    My.Settings.encoderIOMode = device.IOMode
                    My.Settings.isEncoderIOModeSet = True
                End If
            Next
            My.Settings.Save()
            updateGUI(device)
        ElseIf state = 1 Then 'reverting to saved settings
            outputText = "Reverted to saved settings!"
            applySavedSettings(device, collection)
        ElseIf state = 2 Then 'reverting to default settings
            outputText = "Reverted to default settings!"
            setDefaults(device, collection)
        End If

        If collection.Count > 0 Then
            MessageBox.Show(outputText)
        End If
    End Sub
#End Region

    Private Sub setDataPoints_Click(sender As Object, e As EventArgs) Handles setDataPoints.Click
        Dim result As Integer
        If Integer.TryParse(dataPointsText.Text, result) Then
            If result > 0 Then 'can't have negative increment value
                dataPoints = result
                My.Settings.dataPoints = result
                My.Settings.isDataPointsSet = True
                My.Settings.Save()
            End If
        End If
    End Sub
End Class
